<?php
/**
 * Plugin Name:       Lockee
 * Plugin URI:        https://wordpress.lockee.fr/
 * Description:       Add Lockee padlock to Wordpress
 * Version:           1.0.0
 * Author:            Nicolas Desmarets
 * Author URI:        https://lockee.fr/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       lockee
**/

function lockee_admin_includes(){
	wp_enqueue_style('leaflet', plugins_url('/css/leaflet.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_style('styles', plugins_url('/css/styles.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_style('light', plugins_url('/css/light.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_script('leaflet', plugins_url('/js/leaflet.js', __FILE__), false, '1.0.0', false);
	wp_enqueue_script('voca', plugins_url('/js/voca.js', __FILE__), array('wp-i18n'), '1.0.0', false);
	wp_enqueue_script('lock', plugins_url('/js/lock.js', __FILE__), array('jquery'), '1.0.0', false);
	$script  = 'templateUrl = "'. plugin_dir_url( __FILE__ ) .'";';
	wp_add_inline_script('lock', $script, 'before');
}

function lockee_includes(){
	wp_enqueue_style('google_fonts', 'https://fonts.googleapis.com/css?family=Nunito+Sans:400,700&display=swap');
	wp_enqueue_style( 'leaflet', plugins_url('/css/leaflet.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_style( 'styles', plugins_url('/css/styles.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_style( 'light', plugins_url('/css/light.css', __FILE__), false, '1.0.0', 'all');
	wp_enqueue_script('leaflet', plugins_url('/js/leaflet.js', __FILE__), false, '1.0.0', false);
	wp_enqueue_script('voca', plugins_url('/js/voca.js', __FILE__), array('wp-i18n'), '1.0.0', false);
	wp_enqueue_script('lock', plugins_url('/js/lock.js', __FILE__), array('jquery'), '1.0.0', false);
	$script  = 'templateUrl = "'. plugin_dir_url( __FILE__ ) .'";';
	wp_add_inline_script('lock', $script, 'before');
	wp_enqueue_script('func', plugins_url('/js/func.js', __FILE__), array('jquery'), '1.0.0', true);
	$script  = 'ajaxurl = "'. plugin_dir_url( __FILE__ ).'fast-ajax.php";';
	wp_add_inline_script('func', $script, 'before');
}

if(!function_exists('lockee_verifcode')) {
	function lockee_verifcode(){
		if((!empty($_GET["id"]))&&($_GET["code"] != "")){
			$id = intval($_GET['id']);
			$code = htmlspecialchars($_GET['code']);
			$lock_code = get_post_meta($id , 'lockee_code', true);
			$lock_type = get_post_meta($id , 'lockee_type', true);
			$lock_options = get_post_meta($id , 'lockee_options', true);
			$content = apply_filters('the_content', get_the_content(null, false, $id));
			$open = 1;
			
			if($lock_type == "G1"){
				$tcode1 = explode(";", $code);
				$tcode2 = explode(";", $lock_code);
				$earth_radius = 6378137;
				if(($tcode1[0] != $tcode2[0])||($tcode1[1] != $tcode2[1])){
					$rla1 = deg2rad($tcode1[0]);
					$rlo1 = deg2rad($tcode1[1]);
					$rla2 = deg2rad($tcode2[0]);
					$rlo2 = deg2rad($tcode2[1]);
					$dlo = ($rlo2 - $rlo1) / 2;
					$dla = ($rla2 - $rla1) / 2;
					$a = (sin($dla) * sin($dla)) + cos($rla1) * cos($rla2) * (sin($dlo) * sin($dlo));
					$d = 2 * atan2(sqrt($a), sqrt(1 - $a));
					$dist = ($earth_radius * $d);
					if($dist >= min($tcode1[2] + $tcode2[2], 50)){
						$open = 0;
					}
				}
			} else if($lock_type == "G2"){
				$tcode1 = explode(";", $code);
				$tcode2 = explode(";", $lock_code);
				$earth_radius = 6378137;
				if(($tcode1[0] != $tcode2[0])||($tcode1[1] != $tcode2[1])){
					$rla1 = deg2rad($tcode1[0]);
					$rlo1 = deg2rad($tcode1[1]);
					$rla2 = deg2rad($tcode2[0]);
					$rlo2 = deg2rad($tcode2[1]);
					$dlo = ($rlo2 - $rlo1) / 2;
					$dla = ($rla2 - $rla1) / 2;
					$a = (sin($dla) * sin($dla)) + cos($rla1) * cos($rla2) * (sin($dlo) * sin($dlo));
					$d = 2 * atan2(sqrt($a), sqrt(1 - $a));
					$dist = ($earth_radius * $d);
					if($dist >= $tcode2[2]){
						$open = 0;
					}
				}
			} else if($lock_type == "P"){
				if(lockee_ignoreTypes($lock_code, $lock_options) != lockee_ignoreTypes($code, $lock_options)){
					$open = 0;
				}
			} else {
				if($lock_code != $code){
					$open = 0;
				}
			}
			if($open == 0){
				$content = "[[LOCKEE_BADCODE]]";
			}
			$file = $content;
		} else {
			$file = "[[LOCKEE_ERROR]]";
		}
		echo $file;
		wp_die();
	}
}

if(!function_exists('lockee_display_metabox')) {
	function lockee_display_metabox($post){
		$lockee_type = get_post_meta($post->ID, 'lockee_type', true);
		$lockee_code = get_post_meta($post->ID, 'lockee_code', true);
		$lockee_options = get_post_meta($post->ID, 'lockee_options', true);
		$lock_types = array(
			"N" => __("Numérique", "lockee"),
			"D" => __("Directionnel", "lockee"),
			"C" => __("À couleurs", "lockee"),
			"M" => __("Musical", "lockee"),
			"S" => __("À schéma", "lockee"),
			"O1" => __("Avec interrupteurs (4x4)", "lockee"),
			"O2" => __("Avec interrupteurs (5x5)", "lockee"),
			"G1" => __("Géolocalisé (réel)", "lockee"),
			"G2" => __("Géolocalisé (virtuel)", "lockee"),
			"L" => __("À connexion", "lockee"),
			"P" => __("Mot de passe", "lockee")
		);
		echo '<div class="lockee" id="lock-admin">';
		echo '<input type="hidden" id="oldtype" value="'.$lockee_type.'" />';
		echo '<select id="inputtype" name="inputtype" onchange="reloadLock();"><option value="">Aucun</option>';
		foreach($lock_types as $k => $v){
			if($lockee_type == $k){
				echo "<option value='$k' selected>$v</option>";
			} else {
				echo "<option value='$k'>$v</option>";
			}
		}
		echo '</select>';
		echo '<div id="isclose">';
		echo '<div style="position:relative;width:320px;margin:auto;">';
		echo '<input type="hidden" id="inputcode" name="inputcode" value="'.$lockee_code.'" />';
		echo '<input type="hidden" id="inputoptions" name="inputoptions" value="'.$lockee_options.'" />';
		echo '<div id="wrapper-code"></div>';
		echo '</div>';
		echo '<script type="text/javascript">';
		echo 'lockAdmin = new Lock("lockAdmin", "admin", "#wrapper-code", "#inputcode", "'.$lockee_type.'", 2);';
		echo 'function reloadLock(){
			lockAdmin = new Lock("lockAdmin", "admin", "#wrapper-code", "#inputcode", document.getElementById("inputtype").value, 2);	
		}';
		echo '</script>';
		echo '</div></div>';
		wp_nonce_field(plugin_basename( __FILE__ ), 'lockee_metabox_nonce' );
	}
}

if(!function_exists('lockee_initialisation_metabox')) {
	function lockee_initialisation_metabox(){
		add_meta_box('lockee', 'Cadenas Lockee', 'lockee_display_metabox', array('post', 'page'), 'normal', 'high');
	}
}

if(!function_exists('lockee_display_content')) {
	function lockee_display_content($content) {
		global $post;
		$id = $post->ID;
		$lockee_type = get_post_meta($id, 'lockee_type', true);
		$lockee_code = get_post_meta($id, 'lockee_code', true);
		if(empty($lockee_type)){
			return $content;
		} else {
			$contentLock = '<div class="lockee" id="lock-'.$id.'"><div id="isclose">
			<div style="position:relative;width:320px;margin:auto;">
				<div id="wrongcode"><div class="alert">'.__("Code incorrect", "lockee").'</div></div>
				<input type="hidden" id="inputid" name="inputid" value="'.$id.'" />
				<input type="hidden" id="inputcode" name="inputcode" value="" />
				<div id="wrapper-code"></div>
			</div>
			<script type="text/javascript">
				lock'.$id.' = new Lock("lock'.$id.'", "'.$id.'", "#wrapper-code", "#inputcode", "'.$lockee_type.'", 1);
			</script>
			</div>
			<div id="isopen"><div onclick="closeLock(\''.$id.'\', lock'.$id.');" style="position:absolute; top:4px; right:0; cursor:pointer;"><img src="'.plugin_dir_url( __FILE__ ).'imgs/close-light.png" id="close-light" alt="'.__("Refermer le cadenas", "lockee").'" title="'.__("Refermer le cadenas", "lockee").'" /><img src="'.plugin_dir_url( __FILE__ ).'imgs/close-dark.png" id="close-dark" alt="'.__("Refermer le cadenas", "lockee").'" title="'.__("Refermer le cadenas", "lockee").'" /></div>
			<div class="title">'.__("Cadenas ouvert !", "lockee").'</div>
			<div id="contentlock"></div>
			</div></div>';
			return $contentLock;
		}
	}
}

if(!function_exists('lockee_save_metabox')) {
	function lockee_save_metabox($post_id){
		if(!wp_verify_nonce($_POST['lockee_metabox_nonce'], plugin_basename(__FILE__))){
			return $post_id;
		}
		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) 
			return $post_id;
		
		if ('page' == $_POST['post_type'] ||  'post' == $_POST['post_type']){
			if ( !current_user_can( 'edit_page', $post_id ) || !current_user_can( 'edit_post', $post_id ))
				return $post_id;
		} 
		
		if ((!empty($_POST['inputtype'])) && (!empty($_POST['inputcode']))) {
			$code = htmlspecialchars($_POST['inputcode']);
			$type = htmlspecialchars($_POST['inputtype']);
			update_post_meta($post_id, 'lockee_type', $type);
			update_post_meta($post_id, 'lockee_code', $code);
			if(!empty($_POST['ignore'])){
				$options = implode($_POST['ignore'],"");
			} else {
				$options = "";
			}
			update_post_meta($post_id, 'lockee_options', $options);
		} else {
			delete_post_meta($post_id, 'lockee_type');
			delete_post_meta($post_id, 'lockee_code');
			delete_post_meta($post_id, 'lockee_options');
		}
	}
}

if(!function_exists('lockee_ignoreTypes')) {
	function lockee_ignoreTypes($txt, $types){
		if(strpos($types, "A") !== false){
			$txt = htmlentities($txt, ENT_NOQUOTES, 'utf-8');
			$txt = preg_replace('#&([A-za-z])(?:acute|cedil|caron|circ|grave|orn|ring|slash|th|tilde|uml);#', '\1', $txt);
			$txt = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $txt);
			$txt = html_entity_decode($txt); 
		}
		if(strpos($types, "P") !== false){
			$txt = preg_replace('/\p{P}/', '', $txt);
			$exceptions = array("’","#\h#u"," ");
			$txt = str_replace($exceptions, '', $txt);
		}
		if(strpos($types, "C") !== false){
			$txt = mb_strtolower($txt, 'UTF-8');
		}
		return $txt;
	}
}

if(!function_exists('lockee_get_excerpt')) {
	function lockee_get_excerpt( $post_excerpt, $post ) {
		$id = $post->ID;
		$lockee_type = get_post_meta($id, 'lockee_type', true);
		if(empty($lockee_type)){
			return "$post_excerpt";
		} else {
			return __("Ce contenu est protégé par un cadenas", "lockee").' : <a href="' . get_permalink( $post->ID ) . '">'.__("cliquez ici pour l'afficher...", "lockee").'</a>';
		}
	}
}

add_action('wp_enqueue_scripts', 'lockee_includes', 15);
add_action('admin_enqueue_scripts', 'lockee_admin_includes', 15);
add_action("wp_fast_ajax_verifcode", "lockee_verifcode");
add_action("wp_fast_ajax_nopriv_verifcode", "lockee_verifcode");
add_action('add_meta_boxes', 'lockee_initialisation_metabox');
add_action('save_post', 'lockee_save_metabox');
add_filter('the_content', 'lockee_display_content', 99);
add_filter('get_the_excerpt', 'lockee_get_excerpt', 99, 2);

?>